#!/bin/bash

function Error() {
  echo ""
  echo "ERROR: $1"
  exit 1
}

function info() {
  echo "[info] $@"
}

xsltproc --version > /dev/null || Error "xsltproc is not installed on your system."

HTML=0
if [[ "$1" = "--html" ]]; then
  HTML=1
  shift
fi
[[ "$#" = 2 ]] || Error "Usage: $0 [--html] <schematron file> <XML file>"
SCHEMATRON="$1"
XML="$2"

[[ -f "$SCHEMATRON" ]] || Error "could not find schematron file '$SCHEMATRON'"
[[ -f "$XML" ]] || Error "could not find XML file '$XML'"

TMPDIR=$(mktemp -d)
[[ -d "$TMPDIR" ]] || TMPDIR="/tmp"
info "Using tmp directory '$TMPDIR' (the intermediate XSLT files will be generated there)"
[[ -d "$TMPDIR" ]] || Error "Could not create or use temporary directory '$TMPDIR'"

INPUT="$SCHEMATRON"
for XSLT in iso_abstract_expand.xsl iso_svrl_for_xslt1.xsl; do
  [[ -f "$XSLT" ]] || Error "File '$XSLT' not found"
  OUTPUT="$TMPDIR/${SCHEMATRON##*/}__${XSLT}"
  xsltproc "$XSLT" "$INPUT" > "$OUTPUT" || Error "xsltproc error when applying '$XSLT': your schematron is problematic."
  info "Produced intermediate file '$OUTPUT' successfully."
  INPUT="$OUTPUT"
done

SVRL="$TMPDIR/report.svrl"
xsltproc "$INPUT" "$XML" > "$SVRL"\
   || Error "Your compiled schematron fails with your XML document."
info "Intermediate validation report: '$SVRL'."
info "Building the final report..."

XSLT=svrl_to_text.xsl
if (( HTML )); then
  info "Outputting to a HTML file."
  XSLT=svrl_to_xhtml.xsl
fi
[[ -f "$XSLT" ]] || Error "File '$XSLT' not found"
echo ""

OUT="$(xsltproc $XSLT $SVRL)"
errors=$(echo "$OUT" | egrep -i -c "validation.*error")
if (( HTML )); then
  REPORT="report.xhtml"
  echo "$OUT" > "$REPORT"
  echo "Final validation report available in '$REPORT'. Please open it in a browser."
else
  echo "$OUT"
fi

if ((errors>0)); then
  echo "There were $errors errors"
  exit 1  # Validation errors
else
  exit 0  # The file is valid under the schematron.
fi
